#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "tdsio.h"

#define TIME_CHECK
#ifdef	TIME_CHECK
/* dangerous macro! */
#define KHz		* 1000
#define MHz		* 1000 * 1000
static const unsigned int Freq[] = {
	( 40 KHz),	(100 KHz),	(200 KHz),	(500 KHz),
	(  1 MHz),	(  2 MHz),	(  4 MHz),	(  8 MHz),
	( 16 MHz),		0,		0,		0,
		0,		0,		0,		0
};
static const unsigned int Bitw[] = {
	1,	2,	4,	8
};
static const unsigned int Chan[] = {
	1,	4
};
#endif	/*TIME_CHECK*/

static const char *progName;

int main(int argc, char **argv)
{
	char *devfile = NULL;
	char *outfile = NULL;
	tdsdev_t *tdsdev = NULL;
	int fd_out = -1;
	mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
	int rv;
	tds_samp_t sampling = { 40, 'k', 1, 1};
	int samplingVerbose = 0;
	int samplingCount = -1;
	int samplingBlock = 0;
#ifdef	TIME_CHECK
	int checkTimeSync = 0;
#endif	/*TIME_CHECK*/
	tds_time_t tdstime;
	u_int32_t year;
	u_int32_t status;
	u_int32_t *buf = NULL;
	int opt;
	extern int optind;
	extern char *optarg;

	/* ޥɹԥץβ */
	progName = argv[0];
	while((opt = getopt(argc, argv, "i:o:n:b:vR:W:C:T")) != EOF) {
		switch(opt) {
		case 'i':	/* input device */
			devfile = optarg;
			break;
		case 'o':	/* output file */
			outfile = optarg;
			break;
		case 'n':	/* repeat count */
			samplingCount = (int)strtol(optarg, NULL, 0);
			break;
		case 'b':	/* read block size */
			samplingBlock = (int)strtoul(optarg, NULL, 0);
			break;
		case 'v':	/* verbose mode */
			samplingVerbose = 1;
			break;
		case 'R': {	/* sampling rate */
			char *p;
			sampling.freq = strtol(optarg, &p, 0);
			sampling.unit = *p;
			break;
		}
		case 'W':	/* sampling bit width */
			sampling.width = strtol(optarg, NULL, 0);
			break;
		case 'C':	/* number of channel */
			sampling.channel = strtol(optarg, NULL, 0);
			break;
		case 'T':
#ifdef	TIME_CHECK
			checkTimeSync = 1;
			break;
#endif	/*TIME_CHECK*/
		case '?':
		default:
			fprintf(stderr, "usage: %s [-i device] [-o file]"
				" [-n repeat] [-b block] [-v]"
#ifdef	TIME_CHECK
				" [-T]"
#endif	/*TIME_CHECK*/
				" [-R rate] [-W bit-width] [-C channes]\n",
				progName);
			return -1;
		}
	}

	/* ǥХΥץ */
	tdsdev = TdsOpen(devfile);
	if (tdsdev == NULL) {
		perror("TdsOpen() failed ");
		return -1;
	}
	else {
		printf("TdsOpen() OK\n");
	}

	/* ϥեΥץ */
	if (outfile != NULL && strcmp(outfile, "-") == 0) {
		fd_out = 1;
	} else if (outfile != NULL) {
		fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
		if (fd_out == -1) {
			perror("output file open() error");
			goto end;
		}
	}

	/* ǥХ */
	rv = TdsInit(tdsdev);
	if (rv != 0) {
		perror("TdsInit() failed ");
		return -1;
	}
	else {
		printf("TdsInit() OK\n");
	}

	/* ץ */
	rv = TdsSetSampling(tdsdev, &sampling);
	if (rv != 0) {
		perror("TdsSetSampling() failed ");
		printf("rv = %d\n", rv);
		goto end;
	}
	else {
		printf("TdsSetSampling() OK\n");
	}

	/* ץ󥰥⡼ɼ */
	rv = TdsGetSampling(tdsdev, &sampling);
	if (rv != 0) {
		perror("TdsGetSampling() failed ");
		goto end;
	}
	else {
		printf("TdsGetSampling() OK\n");
		printf("%d%cHz/%dbit/%dch\n", sampling.freq, sampling.unit,
			   sampling.width, sampling.channel);
	}

	/* ǥХFIFOꥢ */
	rv = TdsClrBuffer(tdsdev);
	if (rv != 0) {
		perror("TdsClrBuffer() failed ");
		goto end;
	}
	else {
		printf("TdsClrBuffer() OK\n");
	}
	/* 1PPSƱ */
	rv = TdsSync1pps(tdsdev);
	if (rv != 0) {
		perror("TdsSync1pps() failed ");
		goto end;
	}
	else {
		printf("TdsSync1pps() OK\n");
	}
	/* Ʊ */
	rv = TdsSyncTime(tdsdev);
	if (rv != 0) {
		perror("TdsSyncTime() failed ");
		/*  */
		rv = TdsSetTime(tdsdev, NULL);
		if (rv != 0) {
			perror("TdsSetTime() failed ");
			goto end;
		}
		else {
			printf("TdsSetTime() OK\n");
		}
	}
	else {
		printf("TdsSyncTime() OK\n");
	}
	/*  */
	rv = TdsGetTime(tdsdev, &tdstime);
	if (rv != 0) {
		perror("TdsGetTime() failed ");
		goto end;
	}
	else {
		printf("TdsGetTime() OK\n");
	}

	rv = TdsGetYear(tdsdev, &year);
	if (rv != 0) {
		int err;
		char *msg;
		perror("TdsGetYear() failed ");
		rv = TdsError(tdsdev, &err, &msg);
		if (rv != 0) {
			perror("TdsError() failed ");
		}
		else {
			printf("TdsError() OK\n");
			printf("ERROR: (%d) %s\n", err, msg);
		}
	}
	else {
		printf("TdsGetYear() OK, year = %d\n", year);
	}

	/* ץ󥰳 */
	rv = TdsStart(tdsdev);
	if (rv != 0) {
		perror("TdsStart() failed ");
		goto end;
	}
	else {
		printf("TdsStart() OK\n");
	}

	rv = TdsStatus(tdsdev, &status);
	if (rv != 0) {
		perror("TdsStatus() failed\n");
		goto end;
	}
	else {
		printf("TdsStatus() OK\n");
	}

	/* ǡХåե */
	if (samplingBlock == 0) {
		int freq = sampling.freq;
		switch(sampling.unit) {
		case 'm': case 'M':	freq *= 1000; /*nobreak!*/
		case 'k': case 'K':	freq *= 1000; /*nobreak!*/
		}
		samplingBlock = freq * sampling.width * sampling.channel
			/ 8 / sizeof(u_int32_t);
		samplingBlock += 2;	/* for time-sync data */
	}
	buf = (u_int32_t *)malloc(samplingBlock * sizeof(u_int32_t));
	if (buf == NULL) {
		perror("malloc() failed ");
		goto end;
	}
	/* ǡɤ߹ */
	{
#ifdef	TIME_CHECK
#define ROUNDUP(val,bound)	((((val) + (bound) - 1) / (bound)) * (bound))
		size_t total_words = 0;
		size_t wordspersec = 0;
		size_t sync_pos;
		u_int32_t last_time;
#endif	/*TIME_CHECK*/
		register int i;
		for(i = 0; i != samplingCount; i++) {
			size_t nbyte = samplingBlock * sizeof(u_int32_t);
			/* TdsGetData() */
			rv = TdsGetData(tdsdev, buf);
			if (rv != 0) {
				perror("TdsGetData() failed ");
				goto end;
			}
			nbyte -= sizeof(u_int32_t);
			/* TdsRead() */
			rv = TdsRead(tdsdev, &nbyte, (unsigned char *)(buf+1));
			if (rv != 0) {
				perror("TdsRead() failed ");
				goto end;
			}
			nbyte += sizeof(u_int32_t);
			if (samplingVerbose) {
				printf(".");
				if ((i % 50) == 49) printf(": %d\n", i+1);
				fflush(stdout);
			}
			/* ǡν񤭽Ф */
			if (fd_out >= 0) {
				rv = write(fd_out, buf, nbyte);
				if (rv != nbyte) {
					perror("write() failed ");
					goto end;
				}
			}
#ifdef	TIME_CHECK
			if (!checkTimeSync)
				continue;
			/* ƱǡΥå */
			if (wordspersec == 0) {
				wordspersec
					= Freq[(buf[1] >> 18) & 0xf]
					* Bitw[(buf[1] >> 22) & 0x3]
					* Chan[(buf[1] >> 17) & 0x1]
					+ 2;
				last_time = (buf[1] & 0x1FFFF) - 1;
			}
			sync_pos = ROUNDUP(total_words, wordspersec);
			while(sync_pos < nbyte / sizeof(u_int32_t)) {
				u_int32_t sync_data = buf[sync_pos+0];
				u_int32_t curr_time = buf[sync_pos+1];
				if (sync_data != 0xffffffff
				 || (curr_time >> 24) != 0x8B) {
					printf("%d: sync pattern not found\n", i);
					goto end;
				}
				curr_time &= 0x1FFFF;
				if (curr_time != last_time + 1) {
					printf("%d: uncontig time-of-day value\n", i);
					goto end;
				}
				last_time = curr_time;
				sync_pos += wordspersec;
			}
			total_words += nbyte / sizeof(u_int32_t);
			total_words %= wordspersec;
#endif	/*TIME_CHECK*/
		}
	}

	/* ץ */
	rv = TdsStop(tdsdev);
	if (rv != 0) {
		perror("TdsStop() failed ");
		goto end;
	}
	else {
		printf("TdsStop() OK\n");
	}

 end:
	/* ϥեΥ */
	if (fd_out >= 0) close(fd_out);
	/* ǥХΥ */
	TdsClose(tdsdev);

	return rv;
}
