#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include "tdsio.h"

#include "common.c"

static const unsigned int Freq[] = {
	( 40 KHz),	(100 KHz),	(200 KHz),	(500 KHz),
	(  1 MHz),	(  2 MHz),	(  4 MHz),	(  8 MHz),
	( 16 MHz),		0,		0,		0,
		0,		0,		0,		0
};
static const unsigned int Bitw[] = {
	1,	2,	4,	8
};
static const unsigned int Chan[] = {
	1,	4
};

static const char *progName;

int main(int argc, char **argv)
{
	const char *infile = NULL;
	FILE *inFp = stdin;
	int verbose = 0;
	int ignore_uncontig = 0;
	struct {
	    unsigned int sync;
	    unsigned int time;
	} tod;
	unsigned int data;
	unsigned int curr_time, last_time;
	size_t nread;
	size_t skip = 0;
	size_t i, count;
	int opt;
	extern int optind;
	extern char *optarg;

	progName = argv[0];
	while((opt = getopt(argc, argv, "i:vf")) != EOF) {
		switch(opt) {
		case 'i':	/* input file */
			infile = optarg;
			break;
		case 'v':
			verbose = 1;
			break;
		case 'f':
			ignore_uncontig = 1;
			break;
		case '?':
		default:
			fprintf(stderr,
				"usage: %s [-i file] [-v] [-f] [file]\n",
				progName);
			return -1;
		}
	}
	argc -= optind;
	argv += optind;
	if (argc > 0) {
	    infile = *argv;
	}

	if (infile == NULL) {
		infile = "(stdin)";
	} else if ((inFp = fopen(infile, "r")) == NULL) {
		fprintf(stderr, "%s: %s: %s\n",
			progName, infile, strerror(errno));
		return -1;
	}
		
	for(i = 0; (nread = fread(&tod, sizeof(tod), 1, inFp)) == 1; i++) {
		if (tod.sync != 0xffffffff || (tod.time >> 24) != 0x8B) {
			fprintf(stderr, ": %d sync pattern not found\n", i);
			break;
		}
		if (skip == 0) {
			unsigned int rate  = Freq[(tod.time >> 18) & 0xf];
			unsigned int width = Bitw[(tod.time >> 22) & 0x3];
			unsigned int chann = Chan[(tod.time >> 17) & 0x1];
			skip  = rate * width * chann / 32;
			last_time = (tod.time & 0x1FFFF) - 1;
			if (verbose) {
				printf("sampling: %dHz/%dbit/%dCH\n",
					rate, width, chann);
				printf("bytes/1pps: %d\n", (skip+2) * 4);
				printf("start-time: %6d\n", (tod.time & 0x1FFFF));
			}
		}
		curr_time = (tod.time & 0x1FFFF);
		if (curr_time != last_time + 1) {
			fprintf(stderr, ": %d uncontig time-of-day value\n", i);
			if (!ignore_uncontig)
				break;
		}
		last_time = curr_time;
		if (verbose) {
			printf(".");
			if ((i % 50) == 49) printf(": %d\n", i+1);
			fflush(stdout);
		}
		for(count = skip; count > 0; count--) {
			if ((nread = fread(&data, sizeof(data), 1, inFp)) != 1)
				break;
		}
	}
	if (nread < 0) {
	    fprintf(stderr, "%s: %s: %s\n", progName, infile, strerror(errno));
	    return -1;
	}
	return 0;
}
