#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include "tdsio.h"

#include "common.c"
#define TIME_CHECK
#ifdef	TIME_CHECK
static const unsigned int Freq[] = {
	( 40 KHz),	(100 KHz),	(200 KHz),	(500 KHz),
	(  1 MHz),	(  2 MHz),	(  4 MHz),	(  8 MHz),
	( 16 MHz),		0,		0,		0,
		0,		0,		0,		0
};
static const unsigned int Bitw[] = {
	1,	2,	4,	8
};
static const unsigned int Chan[] = {
	1,	4
};
#endif	/*TIME_CHECK*/

static const char *progName;

int main(int argc, char **argv)
{
	const char *devfile = "/dev/tds0";
	const char *outfile = "./tds.data";
	int fd_in;
	int fd_out;
	mode_t ofilemode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
	int rv;
	u_int8_t sampling;
	u_int32_t *buf;
	u_int32_t status;
	u_int32_t tdstime;
	int samplingRate = (1 MHz);		/* 1MHz */
	int samplingWidth = 1;			/* 1bit/sample */
	int samplingChannels = 1;		/* 1CH */
	int samplingVerbose = 0;
	int samplingCount = -1;
	int samplingBlock = 0;
#ifdef	TIME_CHECK
	int checkTimeSync = 0;
#endif	/*TIME_CHECK*/
	int opt;
	extern int optind;
	extern char *optarg;

	/* ޥɹԥץβ */
	progName = argv[0];
	while((opt = getopt(argc, argv, "i:o:n:b:vR:W:C:T")) != EOF) {
		switch(opt) {
		case 'i':	/* input device */
			devfile = optarg;
			break;
		case 'o':	/* output file */
			outfile = optarg;
			break;
		case 'n':	/* repeat count */
			samplingCount = (int)strtol(optarg, NULL, 0);
			break;
		case 'b':	/* read block size */
			samplingBlock = (int)strtoul(optarg, NULL, 0);
			break;
		case 'v':	/* verbose mode */
			samplingVerbose = 1;
			break;
		case 'R':	/* sampling rate */
			samplingRate = get_rate(optarg);
			break;
		case 'W':	/* sampling bit width */
			samplingWidth = get_width(optarg);
			break;
		case 'C':	/* number of channel */
			samplingChannels = get_channels(optarg);
			break;
		case 'T':
#ifdef	TIME_CHECK
			checkTimeSync = 1;
			break;
#endif	/*TIME_CHECK*/
		case '?':
		default:
			fprintf(stderr, "usage: %s [-i device] [-o file]"
				" [-n repeat] [-b block] [-v]"
#ifdef	TIME_CHECK
				" [-T]"
#endif	/*TIME_CHECK*/
				" [-R rate] [-W bit-width] [-C channes]\n",
				progName);
			return -1;
		}
	}

	/* ǥХΥץ */
	fd_in = open(devfile, O_RDONLY);
	if (fd_in == -1) {
		perror("device open() error ");
		return -1;
	}

	/* ϥեΥץ */
	if (strcmp(outfile, "-") == 0) {
		fd_out = 1;
	} else if (outfile != NULL) {
		fd_out = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, ofilemode);
		if (fd_out == -1) {
			perror("output file open() error");
			goto end;
		}
	}

	/* ץ */
	sampling = GetSamplingFlag(samplingRate, samplingWidth, samplingChannels);
	rv = ioctl(fd_in, TDSIO_SET_SAMPLING, &sampling);
	if (rv == -1) {
		perror("ioctl() sampling set failed ");
		goto end;
	}
	else {
		printf("ioctl() sampling set succeed\n");
	}

	/* ǥХFIFOꥢ */
	rv = ioctl(fd_in, TDSIO_BUFFER_CLEAR);
	if (rv == -1) {
		perror("ioctl() FIFO clear failed ");
		goto end;
	}
	else {
		printf("ioctl() FIFO clear succeed\n");
		status = 0;
		rv = ioctl(fd_in, TDSIO_GET_STATUS, &status);
		if (rv == -1) {
			perror("ioctl() get status failed ");
			goto end;
		}
		else {
			if (!(status & TDS_STAT_FIFO_EMPTY)) {
				perror("but FIFO is not empty !!!\n");
			}
		}
	}

	/* 1PPSƱ */
	rv = ioctl(fd_in, TDSIO_SYNC_1PPS);
	if (rv == -1) {
		perror("ioctl() 1pps sync failed ");
		goto end;
	}
	else {
		printf("ioctl() 1pps sync succeed\n");
	}

	/* Ʊ */
	rv = ioctl(fd_in, TDSIO_SYNC_TIME);
	if (rv == -1) {
		time_t ttt;
		struct tm *ptm;
		unsigned int sec, day, date;
		perror("ioctl() time sync failed ");
		(void)time(&ttt);
		ptm = localtime(&ttt);
		if (ptm == NULL) {
			perror("localtime() failed ");
			goto end;
		}
		sec = (ptm->tm_hour * 3600) + (ptm->tm_min * 60) + ptm->tm_sec;
		day = ptm->tm_yday;
		printf("time(yday:sec) set to %d:%d\n", day, sec);
		date = TDS_MAKE_TIME(day, sec);
		rv = ioctl(fd_in, TDSIO_SET_TIME, &date);
		if (rv == -1) {
			perror("ioctl() time set failed ");
			goto end;
		}
	}
	else {
		printf("ioctl() time sync succeed\n");
	}
	/*  */
	rv = ioctl(fd_in, TDSIO_GET_TIME, &tdstime);
	if (rv == -1) {
		perror("ioctl() get time failed ");
	}
	else {
		printf("TIME : 0x%08x\n", tdstime);
	}

	/* ץ󥰳 */
	rv = ioctl(fd_in, TDSIO_SAMPLING_START);
	if (rv == -1) {
		perror("ioctl() start sampling failed ");
		goto end;
	}
	else {
		printf("ioctl() start sampling succeed\n");
	}

	/* ǡХåե */
	if (samplingBlock == 0) {
		samplingBlock = samplingRate * samplingWidth * samplingChannels
			/ 8 / sizeof(u_int32_t);
		samplingBlock += 2;	/* for time-sync data */
	}
	buf = (u_int32_t *)malloc(samplingBlock * sizeof(u_int32_t));
	if (buf == NULL) {
		perror("malloc() failed ");
		goto end;
	}
	/* ǡɤ߹ */
	{
#ifdef	TIME_CHECK
#define ROUNDUP(val,bound)	((((val) + (bound) - 1) / (bound)) * (bound))
		size_t total_words = 0;
		size_t wordspersec = 0;
		size_t sync_pos;
		u_int32_t last_time;
#endif	/*TIME_CHECK*/
		register int i;
		for(i = 0; i != samplingCount; i++) {
			size_t nbyte = samplingBlock * sizeof(u_int32_t);
			rv = read(fd_in, buf, nbyte);
			if (rv < 0) {
				perror("read() failed ");
				goto end;
			}
			nbyte = rv;
			if (samplingVerbose) {
				printf(".");
				if ((i % 50) == 49) printf(": %d\n", i+1);
				fflush(stdout);
			}
			/* ǡν񤭽Ф */
			rv = write(fd_out, buf, nbyte);
			if (rv != nbyte) {
				perror("write() failed ");
				goto end;
			}
#ifdef	TIME_CHECK
			if (!checkTimeSync)
			    continue;
			/* ƱǡΥå */
			if (wordspersec == 0) {
				wordspersec
					= Freq[(buf[1] >> 18) & 0xf]
					* Bitw[(buf[1] >> 22) & 0x3]
					* Chan[(buf[1] >> 17) & 0x1]
					+ 2;
				last_time = (buf[1] & 0x1FFFF) - 1;
			}
			sync_pos = ROUNDUP(total_words, wordspersec);
			while(sync_pos < nbyte / sizeof(u_int32_t)) {
				u_int32_t sync_data = buf[sync_pos+0];
				u_int32_t curr_time = buf[sync_pos+1];
				if (sync_data != 0xffffffff
				 || (curr_time >> 24) != 0x8B) {
					printf("%d: sync pattern not found\n", i);
					goto end;
				}
				curr_time &= 0x1FFFF;
				if (curr_time != last_time + 1) {
					printf("%d: uncontig time-of-day value\n", i);
					goto end;
				}
				last_time = curr_time;
				sync_pos += wordspersec;
			}
			total_words += nbyte / sizeof(u_int32_t);
			total_words %= wordspersec;
#endif	/*TIME_CHECK*/
		}
	}

	/* ץ */
	rv = ioctl(fd_in, TDSIO_SAMPLING_STOP);
	if (rv == -1) {
		perror("ioctl() stop sampling failed ");
		goto end;
	}
	else {
		printf("ioctl() stop sampling succeed\n");
	}

 end:
	/* ϥեΥ */
	close(fd_out);
	/* ǥХΥ */
	close(fd_in);

	return rv;
}
