#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

typedef	unsigned int	u_int32_t;

static void error(int status, const char *format, ...);
static int open_socket(int flags, int family, int socktype, int protocol,
	const char *nodeName, const char *portName);
static const char *progName;

struct tds_client_t {
	int fd;
	struct sockaddr	addr;
	socklen_t	len;
};
#define MAX_CLIENTS	1
static struct tds_client_t	client[MAX_CLIENTS];

int main(int argc, char **argv)
{
	const char *outputFile = "/dev/null";
	const char *portName = "56789";
	const char *nodeName = NULL;
	int family = PF_INET;
	int socktype = SOCK_DGRAM;
	int protocol = IPPROTO_UDP;
	const char *protoName = "udp";
	int fd_in, fd_out;
	u_int32_t *buffer;
	size_t bufsiz = BUFSIZ;

	int opt;
	extern int optind;
	extern char *optarg;

	progName = argv[0];
	while((opt = getopt(argc, argv, "p:h:o:TU")) != EOF) {
		switch(opt) {
		case 'p':	/* port number/name */
			portName = optarg;
			break;
		case 'h':	/* host address/name */
			nodeName = optarg;
			break;
		case 'o':	/* output filename */
			outputFile = optarg;
			break;
		case 'T':	/* use TCP */
			socktype = SOCK_STREAM;
			protocol = IPPROTO_TCP;
			protoName = "tcp";
			break;
		case 'U':	/* use UDP */
			socktype = SOCK_DGRAM;
			protocol = IPPROTO_UDP;
			protoName = "udp";
			break;
		case '?':
		default:
			error(-1, "usage: %s [-p port] [-t target] [-o file] [-T|-U]",
				progName);
		}
	}
	argc -= optind;
	argv += optind;
	if (argc > 0) {
		nodeName = argv[0];
	}

	/* ϥåȤΥץ */
	fd_in = open_socket(AI_PASSIVE, family, socktype, protocol,
		nodeName, portName);
	if (fd_in < 0) {
		error(-1, "socket(): %s", strerror(errno));
	}

	/* ϥեΥץ */
#define OPEN_FLAG	(O_WRONLY | O_CREAT | O_TRUNC)
#define OPEN_MODE	(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
	fd_out = open(outputFile, OPEN_FLAG, OPEN_MODE);
	if (fd_out < 0) {
		error(-1, "open(): %s: %s", outputFile, strerror(errno));
	}

	/* Хåեݤ */
	if ((buffer = (u_int32_t *)malloc(bufsiz * sizeof(u_int32_t))) == NULL) {
		error(-1, "malloc(): %s", strerror(errno));
	}

	/* */
	if (protocol == IPPROTO_TCP) {
		if (listen(fd_in, 5) < 0) {
			error(-1, "listen(): %s: %s", "SOCKET", strerror(errno));
		}
	}
	/* ǡμ */
	while(1) {
		fd_set readfds;
		int nfds, rv;
		register int i;

		FD_ZERO(&readfds);
		FD_SET(fd_in, &readfds);
		nfds = fd_in;
		for(i = 0; i < MAX_CLIENTS; i++) {
			if (client[i].len) {
				FD_SET(client[i].fd, &readfds);
				if (nfds < client[i].fd)
					nfds = client[i].fd;
			}
		}
		if ((rv = select(nfds+1, &readfds, NULL, NULL, NULL)) < 0) {
			error(0, "select(): %s", strerror(errno));
			break;
		} else
		if (rv == 0) {
			error(0, "select(): %s", "timeout");
			continue;
		}
		if (FD_ISSET(fd_in, &readfds)) {
			if (protocol == IPPROTO_TCP) {
				struct sockaddr addr_rec;
				socklen_t addr_len;
				int new_fd;
				if ((new_fd = accept(fd_in, &addr_rec, &addr_len)) < 0) {
					error(0, "accepr(): %s", strerror(errno));
					break;
				}
				for(i = 0; i < MAX_CLIENTS; i++) {
					if (client[i].len == 0) {
						client[i].fd = new_fd;
						client[i].addr = addr_rec;
						client[i].len = addr_len;
						break;
					}
				}
			} else if (process(fd_in, fd_out, buffer, bufsiz) < 0) {
				break;
			}
		}
		for(i = 0; i < MAX_CLIENTS; i++) {
			if (FD_ISSET(client[i].fd, &readfds)) {
				if ((rv = process(client[i].fd, fd_out, buffer, bufsiz)) < 0) {
					break;
				} else if (rv == 0) {
					close(client[i].fd), client[i].fd = -1;
					client[i].len = 0;
				}
			}
		}
	}
	free(buffer);

	/* ϥեΥ */
	close(fd_out);
	/* ǥХΥ */
	close(fd_in);
	return 0;
}

static int process(int fd_in, int fd_out, u_int32_t *buffer, size_t bufsiz)
{
	size_t nbytes = read(fd_in, buffer, bufsiz);
	if (nbytes < 0) {
		error(0, "read(): %s: %s", "SOCKET", strerror(errno));
		return nbytes;
	} else
	if (nbytes == 0) {
		error(0, "read(): %s: EOF detected", "SOCKET");
		return nbytes;
	}
	if (write(fd_out, buffer, nbytes) != nbytes) {
		error(0, "read(): %s: %s", "SOCKET", strerror(errno));
		return -1;
	}
	return nbytes;
}

static void error(int status, const char *format, ...)
{
	va_list ap;
	va_start(ap, format);
	 fprintf(stderr, "%s: ", progName);
	vfprintf(stderr, format, ap);
	 fprintf(stderr, "\n");
	va_end(ap);
	if (status) exit(status);
}

static int open_socket(int flags, int family, int socktype, int protocol,
	const char *nodeName, const char *servName)
{
	struct addrinfo hints, *res = NULL;
	int sock, on, err;

	/* ɥ쥹󡢥ݡȾ */
	memset(&hints, 0, sizeof(hints));
	hints.ai_flags    = flags;
	hints.ai_family   = family;
	hints.ai_socktype = socktype;
	hints.ai_protocol = protocol;
	if ((err = getaddrinfo(nodeName, servName, &hints, &res)) != 0) {
		error(-1, "getaddrinfo() failed", gai_strerror(err));
	}

	/* åȤ򥪡ץ󤹤 */
	sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
	if (sock < 0) {
		freeaddrinfo(res);
		error(-1, "socket() failed: %s", strerror(errno));
	}
	/* ɥ쥹κѤĤ */
	on = 1;
	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
		freeaddrinfo(res);
		close(sock);
		error(-1, "setsockopt() failed: %s", strerror(errno));
	}
	/* åȤ˥ɥ쥹Хɤ */
	if (bind(sock, res->ai_addr, res->ai_addrlen) < 0) {
		freeaddrinfo(res);
		close(sock);
		error(-1, "bind(): %s", strerror(errno));
	}

	freeaddrinfo(res);
	return sock;
}
