/* $Id: tdsio.h,v 1.2 2002/06/20 13:51:27 fujita Exp $ */

#ifndef _LINUX_TDSIO_H_
#define _LINUX_TDSIO_H_


#include <asm/ioctl.h>


/* sampling control */
#define TDS_SAMPLING_START	0x01
#define TDS_SAMPLING_STOP	0x00


/* interrupt */
#define TDS_INTR_NONE		0
#define TDS_INTR_1PPS		(0x01 << 0)
#define	TDS_INTR_DMA_DONE	(0x01 << 1)
#define TDS_INTR_DATA_EXIST	(0x01 << 3)
#define TDS_INTR_DATA_EMPTY	(0x01 << 4)
#define TDS_INTR_DATA_FULL	(0x01 << 5)
#define TDS_INTR_DATA_HALF	(0x01 << 6)	/* XXX: works fine ? */


/* sampling freqency */
#define TDS_MASK_SAMP_FREQ	0x0f
#define TDS_GET_SAMP_FREQ(arg)	((arg) & TDS_MASK_SAMP_FREQ)
#define TDS_SAMPLING_40KHZ	0x00
#define TDS_SAMPLING_100KHZ	0x01
#define TDS_SAMPLING_200KHZ	0x02
#define TDS_SAMPLING_500KHZ	0x03
#define TDS_SAMPLING_1MHZ	0x04
#define TDS_SAMPLING_2MHZ	0x05
#define TDS_SAMPLING_4MHZ	0x06
#define TDS_SAMPLING_8MHZ	0x07
#define TDS_SAMPLING_16MHZ	0x08


/* sampling width */
#define TDS_MASK_SAMP_WIDTH	(0x03 << 4)
#define TDS_GET_SAMP_WIDTH(arg)	((arg) & TDS_MASK_SAMP_WIDTH)
#define TDS_SAMPLING_1BIT	(0x00 << 4)
#define TDS_SAMPLING_2BIT	(0x01 << 4)
#define TDS_SAMPLING_4BIT	(0x02 << 4)
#define TDS_SAMPLING_8BIT	(0x03 << 4)


/* sampling channel */
#define TDS_MASK_SAMP_CH	(0x01 << 6)
#define TDS_GET_SAMP_CH(arg)	((arg) & TDS_MASK_SAMP_CH)
#define TDS_SAMPLING_1CH	(0x00 << 6)
#define TDS_SAMPLING_4CH	(0x01 << 6)


/* time sync request / input signal check */
#define TDS_1PPS_SYNC		(0x01 << 0)
#define TDS_TIME_SYNC		(0x01 << 1)
#define TDS_SIGNAL_CHECK	(0x01 << 2)


/* board status */
#define TDS_STAT_10MHZ_INPUT	(0x01 << 0)
#define TDS_STAT_1PPS_INPUT	(0x01 << 1)
#define TDS_STAT_TIME_INPUT	(0x01 << 2)
#define TDS_STAT_SAMPLING	(0x01 << 3)
#define TDS_STAT_FIFO_EMPTY	(0x01 << 4)
#define TDS_STAT_FIFO_FULL	(0x01 << 5)
#define TDS_STAT_FIFO_HALF	(0x01 << 6)
#define TDS_STAT_FIFO_OVERFLOW	(0x01 << 7)


/* data input status */
#define	TDS_INPUT_MAIN		(0x01 << 0)
#define TDS_INPUT_RES1		(0x01 << 1)		/* reserve */
#define TDS_INPUT_RES2		(0x01 << 2)		/* reserve */
#define TDS_EXTCARD_EXIST	(0x01 << 3)
#define TDS_INPUT_EXT1		(0x01 << 4)
#define TDS_INPUT_EXT2		(0x01 << 5)
#define TDS_INPUT_EXT3		(0x01 << 6)
#define TDS_INPUT_EXT4		(0x01 << 7)


/* TDS driver mode */
#define TDS_MODE_TIMER_DRIVEN	1
#define TDS_MODE_INTR_DRIVEN	2
#define TDS_MODE_DEFAULT	TDS_MODE_INTR_DRIVEN


/* program I/O or DMA */
#define	TDS_READ_USE_PIO	0	/* programmed I/O */
#define	TDS_READ_USE_DMA	1	/* bus master transfer */
#define	TDS_READ_DEFAULT	TDS_READ_USE_DMA


/* ioctl() command code */
#define TDS_IO_GROUP			'N'
#define TDSIO_INIT			_IO (TDS_IO_GROUP, 0x01)
#define TDSIO_BUFFER_CLEAR		_IO (TDS_IO_GROUP, 0x02)
#define TDSIO_SYNC_1PPS			_IO (TDS_IO_GROUP, 0x03)
#define TDSIO_SYNC_TIME			_IO (TDS_IO_GROUP, 0x04)
#define TDSIO_SET_INTERRUPT		_IOW(TDS_IO_GROUP, 0x05, u_int32_t *)	/* obsolete */
#define TDSIO_GET_INTERRUPT		_IOR(TDS_IO_GROUP, 0x06, u_int32_t *)	/* obsolete */
#define TDSIO_SET_TIME			_IOW(TDS_IO_GROUP, 0x07, u_int32_t *)
#define TDSIO_GET_TIME			_IOR(TDS_IO_GROUP, 0x08, u_int32_t *)
#define TDSIO_GET_YEAR			_IOR(TDS_IO_GROUP, 0x09, u_int32_t *)
#define TDSIO_SET_SAMPLING		_IOW(TDS_IO_GROUP, 0x0a, u_int32_t *)
#define TDSIO_GET_SAMPLING		_IOR(TDS_IO_GROUP, 0x0b, u_int32_t *)
#define TDSIO_GET_STATUS		_IOR(TDS_IO_GROUP, 0x0c, u_int32_t *)
#define TDSIO_SAMPLING_START		_IO (TDS_IO_GROUP, 0x0d)
#define TDSIO_SAMPLING_STOP		_IO (TDS_IO_GROUP, 0x0e)
#define TDSIO_GET_ERROR			_IOR(TDS_IO_GROUP, 0x0f, u_int32_t *)
#define TDSIO_SET_VERBOSE		_IOW(TDS_IO_GROUP, 0x11, int *)
#define TDSIO_SET_MODE			_IOW(TDS_IO_GROUP, 0x12, int *)			/* obsolete */
#define TDSIO_GET_MODE			_IOR(TDS_IO_GROUP, 0x13, int *)			/* obsolete */
#define TDSIO_SET_ALLOW_OVERFLOW	_IOW(TDS_IO_GROUP, 0x14, int *)
#define TDSIO_GET_OVERFLOW_COUNT	_IOR(TDS_IO_GROUP, 0x15, int *)
#define	TDSIO_SET_READ_MODE		_IOW(TDS_IO_GROUP, 0x16, int *)
#define TDSIO_GET_READ_MODE		_IOR(TDS_IO_GROUP, 0x17, int *)
#define	TDSIO_GET_INPUT_STATUS		_IOR(TDS_IO_GROUP, 0x18, int *)
#define	TDSIO_GET_DMA_STATUS		_IOR(TDS_IO_GROUP, 0x19, int *)


/* time / date */
#define TDS_SEC_MASK		(0x0001ffff)
#define TDS_SEC_CARRY_MASK	(0x01 << 17)
#define TDS_DAYS_MASK		(0x01ff << 18)
#define TDS_DAYS_CARRY_MASK	(0x01 << 27)
#define TDS_GET_SEC(arg)	((arg) & TDS_SEC_MASK)
#define TDS_GET_SEC_CARRY(arg)	(((arg) & TDS_SEC_CARRY_MASK) >> 17)
#define TDS_GET_DAYS(arg)	(((arg) & TDS_DAYS_MASK) >> 18)
#define TDS_GET_DAYS_CARRY(arg)	(((arg) & TDS_DAYS_CARRY_MASK) >> 27)
#define TDS_MAKE_TIME(day,sec)	\
	((((day) << 18) & TDS_DAYS_MASK) | ((sec) & TDS_SEC_MASK))


/* error code for tds */
#define TDS_ERROR_NO_ERROR		0
#define TDS_ERROR_UNKNOWN		1
#define TDS_ERROR_NOT_SUPPORTED		2
#define TDS_ERROR_NOT_STARTED		3
#define TDS_ERROR_ALREADY_STARTED	4
#define TDS_ERROR_NOT_SET_SAMPLING	5
#define TDS_ERROR_FIFO_OVERFLOW		6
#define TDS_ERROR_1PPS_NOT_DETECT	7
#define TDS_ERROR_10MHZ_NOT_DETECT	8
#define TDS_ERROR_TIME_NOT_DETECT	9
#define TDS_ERROR_NOT_SYNC_1PPS		10
#define TDS_ERROR_NOT_SYNC_TIME		11
#define TDS_ERROR_READ_BUSY		12
#define TDS_ERROR_EXTCARD_NOT_EXIST	13
#define TDS_ERROR_NO_DMA_BUFFER		14


#ifdef __KERNEL__


#else	/* __KERNEL__ */


struct tdsdev {
	char *dev;
	int fd;
	int error;
};

typedef struct tdsdev tdsdev_t;


struct tds_time {
	int sec;
	int sec_carry;
	int day;
	int day_carry;
};

typedef struct tds_time tds_time_t;


struct tds_samp {
	int freq;
	int unit;	/* 'k' or 'M' */
	int width;
	int channel;
};

typedef struct tds_samp tds_samp_t;


tdsdev_t *TdsOpen(char *);
int TdsClose(tdsdev_t *);
int TdsInit(tdsdev_t *);
int TdsClrBuffer(tdsdev_t *);
int TdsSync1pps(tdsdev_t *);
int TdsSyncTime(tdsdev_t *);
int TdsSetTime(tdsdev_t *, tds_time_t *);
int TdsGetTime(tdsdev_t *, tds_time_t *);
int TdsGetYear(tdsdev_t *, unsigned int *);
int TdsSetSampling(tdsdev_t *, tds_samp_t *);
int TdsGetSampling(tdsdev_t *, tds_samp_t *);
int TdsStatus(tdsdev_t *, unsigned int *);
int TdsStart(tdsdev_t *);
int TdsStop(tdsdev_t *);
int TdsGetData(tdsdev_t *, unsigned int *);
int TdsRead(tdsdev_t *, int *, unsigned char *);
int TdsError(tdsdev_t *, int *, char **);


#endif	/*  __KERNEL__ */


#endif	/* ! _LINUX_TDSIO_H_ */


/*
Local Variables:
mode: C
tab-width: 8
c-basic-offset: 8
c-indentation-style: K&R
End:
*/
