/*
 * $Id: tdsvar.h,v 1.3 2002/06/23 07:45:03 fujita Exp $
 */

#ifndef	_TDSVAR_
#define	_TDSVAR_


#ifdef __KERNEL__


/* NIHON TSUSHINKI CO, LTD. Vender ID */
#define	NITSUKI_VENDERID		0x7651	/* XXX : temporary */

/* device ID */
#define	NITSUKI_TIME_SYNC_DATA_SAMPLER	0x0002

/* driver name */
#define NITSUKI_TDS_DRIVER_NAME		"tds"

/* device major */
#define TDS_MAJOR                       244


/* Registers (write) */
#define	TDS_REG_WRITE_SAMPLING_START		0x00
#define	TDS_REG_WRITE_FIFO_CLEAR		0x01
#define	TDS_REG_WRITE_INTERRUPT_SET		0x04
#define	TDS_REG_WRITE_SAMPLING_SET		0x05
#define	TDS_REG_WRITE_SYNCRONIZE		0x06
#define	TDS_REG_WRITE_TIME_SET			0x08
#define	TDS_REG_WRITE_DMA_ADDR_SET		0x10
#define	TDS_REG_WRITE_DMA_SIZE_SET		0x14
#define	TDS_REG_WRITE_DMA_START			0x18
#define	TDS_REG_WRITE_DMA_MODE			0x19
#define TDS_REG_WRITE_CLEAR_WRITE_COUNTER	0x20
#define TDS_REG_WRITE_CLEAR_READ_COUNTER	0x24


/* Registers (read) */
#define	TDS_REG_READ_DATA			0x00
#define	TDS_REG_READ_INTERRUPT			0x04
#define	TDS_REG_READ_STATUS			0x06
#define	TDS_REG_READ_YEAR			0x07
#define	TDS_REG_READ_TIME			0x08
#define	TDS_REG_READ_INPUT			0x0c
#define	TDS_REG_READ_DMA_SIZE			0x14
#define	TDS_REG_READ_DMA_STATUS			0x18
#define	TDS_REG_READ_DMA_MODE			0x19
#define	TDS_REG_READ_WRITE_COUNTER		0x20
#define	TDS_REG_READ_READ_COUNTER		0x24


/* interrupt */
#define TDS_INTR_1PPS			(0x01 << 0)
#define TDS_INTR_DMA_DONE		(0x01 << 1)
#define TDS_INTR_DATA_EXIST		(0x01 << 3)
#define TDS_INTR_DATA_EMPTY		(0x01 << 4)
#define TDS_INTR_DATA_FULL		(0x01 << 5)
#define TDS_INTR_DATA_HALF		(0x01 << 6)

/* dma mode */
#define TDS_DMA_MODE_SOFT		(0x00 << 0)
#define TDS_DMA_MODE_HARD		(0x01 << 0)
#define TDS_DMA_MODE_RELATIVE		(0x00 << 1)
#define TDS_DMA_MODE_REWIND		(0x01 << 1)


struct tds_softc;


struct tds_dma_buf {
	unsigned int order;	/* for get_free_pages() */
	unsigned int virtaddr;
	unsigned long busaddr;
	struct tds_softc	*sc;

	size_t	write_offset;
	size_t	read_offset;
	unsigned	flags;
};
typedef struct tds_dma_buf tds_dma_buf_t;


/* buffer flags */
#define TDS_BUFFLAG_READ_WAIT		(0x01 << 0)
#define TDS_BUFFLAG_DATA_READY		(0x01 << 1)
#define TDS_BUFFLAG_WRITE_BUSY		(0x01 << 2)

/* Hardware internal FIFO size */
#define TDS_FIFO_SIZE		(0x01 << 20)		/* 1M(Byte) */

/* max data size on busmaster transfer */
/* For soft trriger mode */
#define TDS_DMA_MAX_WORD	0x20000		/* 128K words */
#define TDS_DMA_MAX_SIZE	(TDS_DMA_MAX_WORD * sizeof(u32))
/* For hard trriger mode */
#define TDS_DMA_MAX_CHUNK	0x10000		/* 64K words */
#define TDS_DMA_MIN_CHUNK	0x00400		/*  1K words */

#define TDS_DMA_SIZE	TDS_FIFO_SIZE	/*(TDS_DMA_MAX_CHUNK * sizeof(u32))*/
#define	TDS_DMA_NSEG	8

#define	TDS_DMA_USE_CHUNK	0		/* dynamic */


/* data structure for each device */
struct tds_softc {
	struct pci_dev *pdev;
	char dev_name[8];

	struct resource *res_io;
	u32 iobase;

	int rev;
	int irq;

	int sampling_freq;
	int sampling_width;
	int sampling_channel;

	int verbose;
	int error;
	int read_mode;
	int max_fifo_overflow;
	u8 sampling_flag;
	u8 intr_flag;

	unsigned is_open : 1;
	unsigned sampling_done : 1;
	unsigned pps_sync_done : 1;
	unsigned time_sync_done : 1;
	unsigned time_set_done : 1;
	unsigned sampling_start : 1;

	/* lock (for block interrupt in top-half) */
	spinlock_t lock;
	struct semaphore sem;

	/* synchronization variables */
	wait_queue_head_t wait_sync_1pps;	/* for 1PPS inerrupt */
	wait_queue_head_t wait_sync_data;	/* for DATA exist inerrupt */
	wait_queue_head_t wait_buffer_ready;	/* for DMA Finish interrupt */
	wait_queue_head_t wait_fifo_underflow;	/* for FIFO empty iterrupt */
	wait_queue_head_t wait_fifo_overhalf;	/* for FIFO half interrupt */
	wait_queue_head_t wait_fifo_overflow;	/* for FIFO full interrupt */

	/* statistics */
	int fifo_overflow_count;

	tds_dma_buf_t	dmabuf[TDS_DMA_NSEG];

	size_t	dmabuf_write;
	size_t	dmabuf_read;
	size_t	dma_buffer_size;	/* per byte */
	size_t	dma_chunk_size;		/* per byte/DMA */
};



#define TDS_MAX_FIFO_OVERFLOW_DEFAULT	16
#define TDS_SAMPLING_DEFAULT	\
	(TDS_SAMPLING_40KHZ | TDS_SAMPLING_1BIT | TDS_SAMPLING_1CH)


/* FIFO counter */
#define TDS_FIFO_COUNTER_MAX    0x40000000
#define TDS_FIFO_COUNTER_MASK   0x3fffffff
#define TDS_FIFO_COUNTER_CLEAR  0x01


/* DMA control */
#define	TDS_DMA_START	0x01
#define	TDS_DMA_STOP	0x00

/* DMA status */
#define TDS_DMA_STATUS	0x01



#define TDS_TURN_ARROUND_MIN	100
#define TDS_TURN_ARROUND_MAX	4000


#endif	/* __KERNEL__ */


#endif	/* _TDSVAR_ */



/*
Local Variables:
mode: C
tab-width: 8
c-basic-offset: 8
c-indentation-style: K&R
End:
*/
